/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: XMLtraceStartLoader.java,v 1.11 2005/05/05 16:35:54 slavescu Exp $
 *
 * Contributors:
 * IBM - Initial API and implementation
 * 
 * $Id: XMLtraceStartLoader.java,v 1.11 2005/05/05 16:35:54 slavescu Exp $
 **********************************************************************/
package org.eclipse.hyades.loaders.trace;

import org.eclipse.hyades.loaders.util.HierarchyContext;
import org.eclipse.hyades.loaders.util.LoadersUtils;
import org.eclipse.hyades.loaders.util.LookupServiceExtensions;
import org.eclipse.hyades.models.hierarchy.HierarchyPackage;
import org.eclipse.hyades.models.trace.TRCProcess;


/**
 * @author slavescu
 *
 */
public class XMLtraceStartLoader extends TraceXMLFragmentLoader {
    //~ Instance fields ----------------------------------------------------------------------------
	
    protected static final String TRACE_ID = "traceId";
    protected static final String CPU_SAMPLE_RESOLUTION = "cpuSampleResolution";
    protected String traceId;
    protected double cpuSampleResolution;

    //~ Methods ------------------------------------------------------------------------------------

    public void addAttribute(String name, String value) {
        switch (LoadersUtils.getHashCode(name)) {
            case TraceConstants.TRACE_ID_int:
                traceId = value;

                break;
            case TraceConstants.CPU_SAMPLE_RESOLUTION_int:
            	cpuSampleResolution = Double.parseDouble(value);

                break;

            default:
                super.addAttribute(name, value);

                break;
            }
    }

    public void addYourselfInContext() {
        //        zeroTime = time;
		if(context.getAgent()!=null && !context.getAgent().eIsSet(HierarchyPackage.eINSTANCE.getTRCAgent_StartTime()))
		{
			context.getAgent().setStartTime(time);
			if(context.getAgent().getAgentProxy()!=null)
				context.getAgent().getAgentProxy().setStartTime(time);
		}

        TRCProcess p = getProcess();

        if (p.getStartTime() == 0) {
            p.setStartTime(createDeltaTime());
        }
        p.setId(traceId);
        p.setCpuSampleResolution(cpuSampleResolution);

        // cleanup the call stack for previous trace (if exists)
        LookupServiceExtensions.getInstance().deregister(context, CallStackPerThread.class);
    }

    public void initialize(HierarchyContext context, String name) {
        super.initialize(context, name);
        traceId = null;
        cpuSampleResolution=0;
    }
}
