package org.eclipse.hyades.logging.events;

import java.util.List;

/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: IExtendedDataElement.java,v 1.14 2005/02/16 22:20:48 qiyanli Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

/**
 * @author rduggan
 *
 * Taken from "Canonical Situation Data Format: The Common Base Event V1.0.1 
 * ACAB.BO0301.0.1"
 * 
 * The ExtendedDataElement allows for user-supplied name-type-value collections to be
 * specified for extensibility purposes.  This is where a user can include any other 
 * attributes not accounted for in the CommonBaseEvent data model.  Collections specified
 * here are assumed to be product specific data.  The named properties can be filtered,
 * searched on, or referenced by the correlation rules. The 'name' is user defined, 
 * however, the nonexclusive list of reserved keywords are as follows: 
 * 
 * RawData - This keyword is indicative of 'as is' data and usually in a format that may
 *           be proprietary to the producer of such data. It may be in any form including
 *           binary.  It is intended to allow the data to be retrieved verbatim, and to
 *           support tools that understand the format of the context.
 * 
 * RootHeader - This keyword  is intended to identify the root ExtendedDataElement for a
 *              hierarchy of the ExtendedDataElements that are defined by the dataRefs. 
 * 
 * @version 1.0
 * @deprecated This class is deprecated as of 2004-05-17. Use the corresponding class available in org.eclipse.hyades.logging.events.cbe*
 */
public interface IExtendedDataElement {

    public static final String TYPE_BYTE = "byte";
    public static final String TYPE_SHORT = "short";
    public static final String TYPE_INT = "int";
    public static final String TYPE_LONG = "long";
    public static final String TYPE_FLOAT = "float";
    public static final String TYPE_DOUBLE = "double";
    public static final String TYPE_STRING = "string";
    public static final String TYPE_BOOLEAN = "boolean";
    public static final String TYPE_DATE_TIME = "dateTime";
    public static final String TYPE_BYTE_ARRAY = "byteArray";
    public static final String TYPE_SHORT_ARRAY = "shortArray";
    public static final String TYPE_INT_ARRAY = "intArray";
    public static final String TYPE_LONG_ARRAY = "longArray";
    public static final String TYPE_FLOAT_ARRAY = "floatArray";
    public static final String TYPE_DOUBLE_ARRAY = "doubleArray";
    public static final String TYPE_STRING_ARRAY = "stringArray";
    public static final String TYPE_BOOLEAN_ARRAY = "booleanArray";
    public static final String TYPE_DATE_TIME_ARRAY = "dateTimeArray";
    public static final String TYPE_HEX_BINARY = "hexBinary";
    public static final String TYPE_NO_VALUE = "noValue";

    public static final int TYPE_BYTE_VALUE = 0;
    public static final int TYPE_SHORT_VALUE = 1;
    public static final int TYPE_INT_VALUE = 2;
    public static final int TYPE_LONG_VALUE = 3;
    public static final int TYPE_FLOAT_VALUE = 4;
    public static final int TYPE_DOUBLE_VALUE = 5;
    public static final int TYPE_STRING_VALUE = 6;
    public static final int TYPE_BOOLEAN_VALUE = 7;
    public static final int TYPE_DATE_TIME_VALUE = 8;
    public static final int TYPE_BYTE_ARRAY_VALUE = 9;
    public static final int TYPE_SHORT_ARRAY_VALUE = 10;
    public static final int TYPE_INT_ARRAY_VALUE = 11;
    public static final int TYPE_LONG_ARRAY_VALUE = 12;
    public static final int TYPE_FLOAT_ARRAY_VALUE = 13;
    public static final int TYPE_DOUBLE_ARRAY_VALUE = 14;
    public static final int TYPE_STRING_ARRAY_VALUE = 15;
    public static final int TYPE_BOOLEAN_ARRAY_VALUE = 16;
    public static final int TYPE_DATE_TIME_ARRAY_VALUE = 17;
    public static final int TYPE_HEX_BINARY_VALUE = 18;
    public static final int TYPE_NO_VALUE_VALUE = 19;

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public void addChildDataElement(IExtendedDataElement element);

    /**
     * Returns the child data elements
     * 
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     * 
     * @return List of Child Data Elements, otherwise an empty list.
     */
    public List getChildDataElements();

    /**
     * Removes a IExtendedDataElement from the child data elements table.
     */
    public void removeChildDataElement(String name);

    /**
     * Removes all IExtendedDataElements from the child data elements table.
     */
    public void clearChildDataElements();

    /**
     * 
     * The array of values for this extended data element as a string representation
     * of the type specified above, excluding hexBinary.  hexBinary values MUST be 
     * defined using the hexValue property.  This is an OPTIONAL property.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a hexBinary data type.
     * 
     * @return List
     */
    public List getValues();

    /**
     * 
     * The array of values for this extended data element as a string representation
     * of the type specified above, excluding hexBinary.  hexBinary values MUST be 
     * defined using the hexValue property.  This is an OPTIONAL property.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a hexBinary data type.
     * 
     * @return String[]
     */
    public String[] getValuesAsStringArray();

    /**
     * The hexValue is an array of bytes that holds the data for any other data type
     * or complexType not in the supported types described above. The hexValue and 
     * values properties are mutually exclusive.  Only one of these properties 
     * can be defined.  This is an OPTIONAL property.
     * 
     * @return char[]
     */
    public char[] getHexValues();

    /**
     * The hexValue is an array of bytes that holds the data for any other data type
     * or complexType not in the supported types described above. The hexValue and 
     * values properties are mutually exclusive.  Only one of these properties 
     * can be defined.  This is an OPTIONAL property.
     * 
     * @return byte[]
     */
    public byte[] getHexValue();

    /**
     * Returns the name.
     * 
     * The name of the extended data element. This name MUST be unique with respect
     * to all other fields in the event.This is a REQUIRED property.
     * 
     * @return String
     */
    public String getName();

    /**
     * Returns the type.
     * 
     * The data type of the values specified in the values property below. Valid types
     * are as follows:
     * 1.  byte
     * 2.  short
     * 3.  int
     * 4.  long
     * 5.  float
     * 6.  double
     * 7.  string
     * 8.  dateTime
     * 9.  duration
     * 10. byteArray
     * 11. shortArray
     * 12. intArray
     * 13. longArray
     * 14. floatArray
     * 15. doubleArray
     * 16. stringArray
     * 17. dateTimeArray
     * 18. durationArray
     * 19. hexBinary
     * The above data types are the only valid types supported by the ExtendedDataElement
     * type. The default value is 'string'.
     * 
     * @return String
     */
    public String getType();

    /**
     * Returns the type as one of the enumerated type values defined in IExtendedDataElement.
     * 
     * @return int
     */
    public int getTypeValue();

    /**
     * Sets the id.
     * 
     * The primary identifier for the element. This property MUST be globally unique.
     * The recommend value for this is either a 128 bit or 256 bit Globally Unique Id 
     * (represented as hex string). Once this value is set it MUST never be changed.
     * This is an OPTIONAL property.
     * 
     * @param id The id to set
     * @deprecated Not required for CBE 1.0.1.
     */
    public void setId(String id);

	/**
	 * Gets the id.
	 * 
	 * The primary identifier for the element. This property MUST be globally unique.
	 * The recommend value for this is either a 128 bit or 256 bit Globally Unique Id 
	 * (represented as hex string). Once this value is set it MUST never be changed.
	 * This is an OPTIONAL property.
	 * 
	 * @return The id
	 * @deprecated Not required for CBE 1.0.1.
	 */
	public String getId();

    /**
     * Sets the name.
     * 
     * The name of the extended data element. This name MUST be unique with respect
     * to all other fields in the event.This is a REQUIRED property.
     * 
     * @param name The name to set
     */
    public void setName(String name);

    /**
     * Sets the type.
     * 
     * The data type of the values specified in the values property below. Valid types
     * are as follows:
     * 1.  byte
     * 2.  short
     * 3.  int
     * 4.  long
     * 5.  float
     * 6.  double
     * 7.  string
     * 8.  dateTime
     * 9.  duration
     * 10. byteArray
     * 11. shortArray
     * 12. intArray
     * 13. longArray
     * 14. floatArray
     * 15. doubleArray
     * 16. stringArray
     * 17. dateTimeArray
     * 18. durationArray
     * 19. hexBinary
     * The above data types are the only valid types supported by the ExtendedDataElement
     * type. The default value is 'string'.
     * 
     * @param type The type to set
     */
    public void setType(String type);

    /**
     * Sets the type. From one of the enumerated values defined in IExtendedDataElement
     * 
     * @param type The type to set
     */
    public void setType(int type);

    /**
     * 
     * The array of values for this extended data element as a string representation
     * of the type specified above, excluding hexBinary.  hexBinary values MUST be 
     * defined using the hexValue property.  This is an OPTIONAL property.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a hexBinary data type.
     * 
     * @param String[] the values to set
     */
    public void setValues(String[] values);

    /**
     * The hexValue is an array of characters that holds the data for any other data type
     * or complexType not in the supported types described above. The hexValue and 
     * values properties are mutually exclusive.  Only one of these properties 
     * can be defined.  This is an OPTIONAL property.
     * 
     * @param char[] the hexadecimal value to set.
     */
    public void setHexValues(char[] hexValues);

    /**
     * The hexValue is an array of bytes that holds the data for any other data type
     * or complexType not in the supported types described above. The hexValue and 
     * values properties are mutually exclusive.  Only one of these properties 
     * can be defined.  This is an OPTIONAL property.
     * 
     * @param byte[] the hexadecimal value to set.
     */
    public void setHexValue(byte[] hexValue);

    /**
     * Get the string value for this extended data element. The method is only valid 
     * if the extended data element is a String. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return String
     */
    public String getString();

    /**
     * Get the dateTime value as a long holding UTC millisecondsfor this extended data element. 
     * The method is only valid if the extended data element is a dateTime. Otherwise an 
     * exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return long
     */
    public long getDateTimeAsLong();

    /**
     * Get the dateTime value for this extended data element. The method is only valid 
     * if the extended data element is a dateTime. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return String
     */
    public String getDateTime();

    /**
     * Get the boolean value for this extended data element. The method is only valid 
     * if the extended data element is a boolean. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return boolean
     */
    boolean getBoolean();

    /**
     * Get the byte value for this extended data element. The method is only valid 
     * if the extended data element is a byte. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return byte
     */
    byte getByte();

    /**
     * Get the short value for this extended data element. The method is only valid 
     * if the extended data element is a short. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return short
     */
    short getShort();

    /**
     * Get the int value for this extended data element. The method is only valid 
     * if the extended data element is a int. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return int
     */
    int getInt();

    /**
     * Get the long value for this extended data element. The method is only valid 
     * if the extended data element is a long. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return long
     */
    long getLong();

    /**
     * Get the float value for this extended data element. The method is only valid 
     * if the extended data element is a float. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return float
     */
    float getFloat();

    /**
     * Get the double value for this extended data element. The method is only valid 
     * if the extended data element is a double. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return double
     */
    double getDouble();

    /**
     * Get the string[] value for this extended data element. The method is only valid 
     * if the event property is a String[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return String[]
     */
    public String[] getStringArray();

    /**
     * Get the dateTime[] value as a long[] holding UTC millisecondsfor this extended data element. 
     * The method is only valid if the extended data element is a dateTime[]. Otherwise an 
     * exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return long[]
     */
    public long[] getDateTimeArrayAsLong();

    /**
     * Get the dateTime[] value for this extended data element. The method is only valid 
     * if the extended data element is a dateTime[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return String[]
     */
    public String[] getDateTimeArray();

    /**
     * Get the boolean value for this extended data element. The method is only valid 
     * if the extended data element is a boolean[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return boolean[]
     */
    boolean[] getBooleanArray();

    /**
     * Get the byte value for this extended data element. The method is only valid 
     * if the extended data element is a byte[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return byte[]
     */
    byte[] getByteArray();

    /**
     * Get the short[] value for this extended data element. The method is only valid 
     * if the extended data element is a short[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return short[]
     */
    short[] getShortArray();

    /**
     * Get the int[] value for this extended data element. The method is only valid 
     * if the extended data element is a int[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return int[]
     */
    int[] getIntArray();

    /**
     * Get the long[] value for this extended data element. The method is only valid 
     * if the extended data element is a long[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return long[]
     */
    long[] getLongArray();

    /** 
     * Get the float[] value for this extended data element. The method is only valid 
     * if the extended data element is a float[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return float[]
     */
    float[] getFloatArray();

    /**
     * Get the double[] value for this extended data element. The method is only valid 
     * if the extended data element is a double[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return double[]
     */
    double[] getDoubleArray();

    /**
     * Get the hexBinary value for this extended data element. The method is only valid 
     * if the extended data element is a hexBinary. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return char[]
     */
    public char[] getHexBinarys();

    /**
     * Get the hexBinary value for this extended data element. The method is only valid 
     * if the extended data element is a hexBinary. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @return byte[]
     */
    public byte[] getHexBinary();

    /**
     * Set the string value for this extended data element. The method is only valid 
     * if the extended data element is a string. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set
     */
    public void setString(String value);

    /**
     * Set the long value for this extended data element. The method is only valid 
     * if the extended data element is a dateTime. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set
     */
    public void setDateTime(long value);

    /**
     * Set the dateTime value for this extended data element. The method is only valid 
     * if the extended data element is a dateTime. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set* @param String the value to set
     */
    public void setDateTime(String value);

    /**
     * Set the boolean value for this extended data element. The method is only valid 
     * if the extended data element is a boolean. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set
     */
    public void setBoolean(boolean value);

    /**
     * Set the byte value for this extended data element. The method is only valid 
     * if the extended data element is a byte. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set
     */
    public void setByte(byte value);

    /**
     * Set the short value for this extended data element. The method is only valid 
     * if the extended data element is a short. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set
     */
    public void setShort(short value);

    /**
     * Set the int value for this extended data element. The method is only valid 
     * if the extended data element is a int. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set
     */
    public void setInt(int value);

    /**
     * Set the long value for this extended data element. The method is only valid 
     * if the extended data element is a long. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set
     */
    public void setLong(long value);

    /**
     * Set the float value for this extended data element. The method is only valid 
     * if the extended data element is a float. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set
     */
    public void setFloat(float value);

    /**
     * Set the double value for this extended data element. The method is only valid 
     * if the extended data element is a double. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set
     */
    public void setDouble(double value);

    /**
     * Set the String[] value for this extended data element. The method is only valid 
     * if the extended data element is a String[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set
     */
    public void setStringArray(String[] values);

    /**
     * Set the long[] value for this extended data element. The method is only valid 
     * if the extended data element is a dateTime[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set
     */
    public void setDateTimeArray(long[] values);

    /**
     * Set the dateTime value for this extended data element. The method is only valid 
     * if the extended data element is a dateTime[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to sett
     */
    public void setDateTimeArray(String[] values);

    /**
     * Set the boolean[] value for this extended data element. The method is only valid 
     * if the extended data element is a boolean[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set
     */
    public void setBooleanArray(boolean[] values);

    /**
     * Set the byte[] value for this extended data element. The method is only valid 
     * if the extended data element is a byte[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set
     */
    public void setByteArray(byte[] values);

    /**
     * Set the short[] value for this extended data element. The method is only valid 
     * if the extended data element is a short[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set
     */
    public void setShortArray(short[] values);

    /**
     * Set the int[] value for this extended data element. The method is only valid 
     * if the extended data element is a int[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set
     */
    public void setIntArray(int[] values);

    /**
     * Set the long[] value for this extended data element. The method is only valid 
     * if the extended data element is a long[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set
     */
    public void setLongArray(long[] values);

    /**
     * Set the float[] value for this extended data element. The method is only valid 
     * if the extended data element is a float[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set
     */
    public void setFloatArray(float[] values);

    /**
     * Set the double[] value for this extended data element. The method is only valid 
     * if the extended data element is a double[]. Otherwise an exception is thrown.
     * 
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     * 
     * @param value  the value to set
     */
    public void setDoubleArray(double[] values);

    /**
     * Set the hexBinary value for this extended data element. The method is only valid 
     * if the extended data element is a hexBinary. Otherwise an exception is thrown.
     * 
     * @param value  the value to set
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     */
    public void setHexBinarys(char[] values);

    /**
     * Set the hexBinary value for this extended data element. The method is only valid 
     * if the extended data element is a hexBinary. Otherwise an exception is thrown.
     * 
     * @param value  the value to set
     * @throws InvalidExtendedDataElementTypeException if the extended data element is a different data type.
     */
    public void setHexBinary(byte[] values);

    /**
     * Resets the object's properties to their initial (e.g. null) state.
     * 
     * @since 1.0.1
     */
    public void init();

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChild(String name);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChild(String name, int type, String value);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChild(String name, int type, String[] values);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChild(String name, String value);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChild(String name, String[] values);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithBooleanArrayValue(String name, boolean[] values);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithBooleanValue(String name, boolean value);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithByteArrayValue(String name, byte[] values);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithByteValue(String name, byte value);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithDateArrayValue(String name, String[] values);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithDateAsLongValue(String name, long value);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithDatesAsLongValue(String name, long[] values);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithDateValue(String name, String value);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithDoubleArrayValue(String name, double[] values);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithDoubleValue(String name, double value);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithFloatArrayValue(String name, float[] values);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithFloatValue(String name, float value);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithHexValue(String name, String value);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithHexValueAsByteArray(String name, byte[] value);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithIntArrayValue(String name, int[] values);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithIntValue(String name, int value);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithLongArrayValue(String name, long[] values);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithLongValue(String name, long value);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithShortArrayValue(String name, short[] values);

    /**
     * Adds a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement addChildWithShortValue(String name, short value);

    /**
     * Gets a child data element
     *  
     * Other extendedDataElement(s) to specify a structured list of
     * child extendedDataElements.  This list allows a reporter to create a
     * hierarchy of extendedDataElements for a specific CommonBaseEvent.
     * This is an OPTIONAL property.
     */
    public IExtendedDataElement getChildDataElement(String name);
}
