/*******************************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: NewActionGroup.java,v 1.9 2005/02/16 22:22:06 qiyanli Exp $
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.test.ui.internal.navigator.action;

import java.util.Iterator;

import org.eclipse.hyades.test.ui.TestUIPlugin;
import org.eclipse.hyades.test.ui.adapter.TestWorkbenchAdapter;
import org.eclipse.hyades.ui.extension.INavigatorContribution;
import org.eclipse.hyades.ui.internal.logicalfolder.LogicalFolder;
import org.eclipse.hyades.ui.internal.navigator.TreeNavigator;
import org.eclipse.hyades.ui.internal.navigator.action.NavigatorActionGroup;
import org.eclipse.hyades.ui.internal.navigator.action.NewObjectAction;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.actions.NewWizardMenu;
import org.eclipse.ui.views.navigator.ResourceNavigatorMessages;

/**
 * Creation group action for the Test Navigator.
 * 
 * @author marcelop
 * @since 0.0.1
 */
public class NewActionGroup 
extends NavigatorActionGroup
{
	private NewObjectAction newTestSuiteAction;
	private NewObjectAction newDeploymentAction;
	
	private NewObjectAction newHyadesObjectAction;

	/**
	 * Constructor for CreateGroupAction
	 * @param testNavigator
	 */
	public NewActionGroup(TreeNavigator testNavigator)
	{
		super(testNavigator);
	}

	/**
	 * @see org.eclipse.hyades.ui.internal.navigator.action.NavigatorActionGroup#createActions()
	 */
	protected void createActions()
	{
		newTestSuiteAction = new NewObjectAction(getNavigator().getSite().getWorkbenchWindow());
		newTestSuiteAction.setText(TestUIPlugin.getString("LBL_TST_SUITE")); //$NON-NLS-1$
		newTestSuiteAction.setToolTipText(newTestSuiteAction.getText());
		newTestSuiteAction.setWizardTitle(TestUIPlugin.getString("WIZ_TST_SUITE_TTL")); //$NON-NLS-1$
		newTestSuiteAction.setWizardCategory("org.eclipse.hyades.test.ui.wizards.new/testSuite"); //$NON-NLS-1$

		newDeploymentAction = new NewObjectAction(getNavigator().getSite().getWorkbenchWindow());
		newDeploymentAction.setText(TestUIPlugin.getString("LBL_DEPLOYMENT")); //$NON-NLS-1$
		newDeploymentAction.setToolTipText(newDeploymentAction.getText());
		newDeploymentAction.setWizardTitle(TestUIPlugin.getString("WIZ_DEPLOY_TTL")); //$NON-NLS-1$
		newDeploymentAction.setWizardCategory("org.eclipse.hyades.test.ui.wizards.new/testElements"); //$NON-NLS-1$
		
		newHyadesObjectAction = new NewObjectAction(getNavigator().getSite().getWorkbenchWindow());
		newHyadesObjectAction.setText(TestUIPlugin.getString("WIZ_TST_OBJ_ACT")); //$NON-NLS-1$
		newHyadesObjectAction.setWizardTitle(TestUIPlugin.getString("WIZ_TST_OBJ_TTL")); //$NON-NLS-1$
		newHyadesObjectAction.setWizardCategory("org.eclipse.hyades.test.ui.wizards.new"); //$NON-NLS-1$
		newHyadesObjectAction.setToolTipText(newHyadesObjectAction.getText());
	}
	
	/**
	 * @see org.eclipse.ui.actions.ActionGroup#fillContextMenu(org.eclipse.jface.action.IMenuManager)
	 */
	public void fillContextMenu(IMenuManager menu)
	{
		IStructuredSelection structuredSelection = (IStructuredSelection)getContext().getSelection();

        
		if(getNavigator().isShowingFolders())
		{
			if(!hasLogicalFolder(structuredSelection))
			{
				MenuManager newMenu = new MenuManager(ResourceNavigatorMessages.getString("ResourceNavigator.new"), INavigatorContribution.GROUP_SUB_NEW); //$NON-NLS-1$
				menu.appendToGroup(INavigatorContribution.GROUP_NEW, newMenu);

				new NewWizardMenu(newMenu, getNavigator().getSite().getWorkbenchWindow(), false);

				newMenu.insert(0, new ActionContributionItem(newHyadesObjectAction));
				newMenu.insert(1,new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
			}
		}
		else
		{
			if(structuredSelection.size() == 1)
			{
				Object selection = structuredSelection.getFirstElement();
				if(selection instanceof LogicalFolder)
				{
					LogicalFolder logicalFolder = (LogicalFolder)selection;
					if(logicalFolder.getParent() == null)
					{
						MenuManager newMenu = new MenuManager(ResourceNavigatorMessages.getString("ResourceNavigator.new")); //$NON-NLS-1$
						menu.appendToGroup(INavigatorContribution.GROUP_NEW, newMenu);

						String name = logicalFolder.getName();
						if(TestWorkbenchAdapter.TEST_SUITE_LOGICAL_FOLDER_NAME.equals(name))
							newMenu.add(newTestSuiteAction);

						else if(TestWorkbenchAdapter.DEPLOYMENT_LOGICAL_FOLDER_NAME.equals(name))
							newMenu.add(newDeploymentAction);
					}
				}
			}
		}
	}
	
	/**
	 * Returns <code>true</code> is there is one logical folder in the 
	 * given selection.
	 * @param structuredSelection
	 * @return boolean
	 */
	protected boolean hasLogicalFolder(IStructuredSelection structuredSelection)
	{
		for (Iterator i = structuredSelection.iterator(); i.hasNext();)
		{
			if(i.next() instanceof LogicalFolder)
				return true;
		}
		
		return false;
	}
}
